// -*- C++ -*-
/*!
 * @file  WiiClassicControllerTranslator.cpp
 * @brief WiiClassicControllerTranslator
 * @date $Date$
 *	
 * @author 〒135-8548
 * 東京都江東区豊洲3-7-5　芝浦工業大学　研究棟　11-Q-32
 * 水川(HRI)研究室
 * 代表者:水川　真   (Emai:mizukawa<at>sic.shibaura-it.ac.jp)
 * 作成者;田中　基雅(Email:m108081<at>shibaura-it.ac.jp)
 *          藤田　恒彦(Email:m109077<at>shibaura-it.ac.jp)
 *          鷹栖　尭大（Email:m109056<at>shibaura-it.ac.jp）
 *
 * WiiClassicControllerTranslatorについての著作権は，芝浦工業大学 水川研究室に帰
 * 属します．
 * 非営利目的利用は自由です．
 *
 * $Id$
 */

#include "WiiClassicControllerTranslator.h"

// Module specification
// <rtc-template block="module_spec">
static const char* wiiclassiccontrollertranslator_spec[] =
  {
    "implementation_id", "WiiClassicControllerTranslator",
    "type_name",         "WiiClassicControllerTranslator",
    "description",       "WiiClassicControllerTranslator",
    "version",           "1.0.0",
    "vendor",            "Shibaura Institute of Technology",
    "category",          "RemoteController",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    //"exec_cxt.periodic.rate", "1.0",
    // Configuration variables
    "conf.default.debug", "0",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
WiiClassicControllerTranslator::WiiClassicControllerTranslator(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_WiiDataIn("WiiData", m_WiiData),
    m_WiiConvertOut("WiiConvert", m_WiiConvert),
    m_WiiInfoPort("WiiInfo")

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
WiiClassicControllerTranslator::~WiiClassicControllerTranslator()
{
}


/*!
 * Configurationの初期化を行う
 */
RTC::ReturnCode_t WiiClassicControllerTranslator::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("WiiData", m_WiiDataIn);
  
  // Set OutPort buffer
  registerOutPort("WiiConvert", m_WiiConvertOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  m_WiiInfoPort.registerConsumer("WiiInfo", "WiiInfo", m_WiiInfo);
  
  // Set CORBA Service Ports
  registerPort(m_WiiInfoPort);
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("debug", m_debug, "0");
  
  // </rtc-template>
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t WiiClassicControllerTranslator::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t WiiClassicControllerTranslator::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t WiiClassicControllerTranslator::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*!
 * キャリブレーションデータを取得し，WiiRemoteAgentに送信する．
 */

RTC::ReturnCode_t WiiClassicControllerTranslator::onActivated(RTC::UniqueId ec_id)
{
	//キャリブレーションデータ格納用
	Calibration_var CalibList;
	//キャリブレーションデータの格納

	BYTE status = 0x00;
	BYTE buff[22];

	while(1)
	{
		CalibList = m_WiiInfo->get_wii_mote_calib();
		status = CalibList[0];
		if(status == 0x21)
		{
			break;
		}
		Sleep(50);
	}

	for(int i = 0; i < 22 ; i++){
		buff[i] = CalibList[i];
	}
	wii.SetWiiRemoteCalibration(buff);

	while(1)
	{
		CalibList = m_WiiInfo->get_wii_ext_calib();
		status = CalibList[0];
		if(status == 0x21)
		{
			break;
		}
		Sleep(50);
	}

	for(int i = 0; i < 22 ; i++){
		buff[i] = CalibList[i];
	}
	wii.SetWiiClassicControllerCalibration(buff);
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t WiiClassicControllerTranslator::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*!
 * WiiRemoteAgentから生データが入力された時に，それを整形データに変換し，出力する
 * ．
 * 整形データはTimedShortSeq型で出力する．
 * データの内訳を以下に記す．
 * 尚，[0]から[13]までがWiiリモコン本体の情報，それ以降はClassicControllerの情報とな
 * っている．
 * [0] 十字キー・上ボタンの状態(BOOL 値)
 * [1] 十字キー・右ボタンの状態(BOOL 値)
 * [2] 十字キー・下ボタンの状態(BOOL 値)
 * [3] 十字キー・左ボタンの状態(BOOL 値)
 * [4] A ボタンの状態(BOOL 値)
 * [5] B ボタンの状態(BOOL 値)
 * [6] ＋ボタンの状態(BOOL 値)
 * [7] Home ボタンの状態(BOOL 値)
 * [8] －ボタンの状態(BOOL 値)
 * [9] 1 ボタンの状態(BOOL 値)
 * [10] 2 ボタンの状態(BOOL 値)
 * [11] X軸方向の加速度
 * [12] Y軸方向の加速度
 * [13] Z軸方向の加速度
 * [14] A ボタンの状態(BOOL 値)
 * [15] B ボタンの状態(BOOL 値)
 * [16] X ボタンの状態(BOOL 値)
 * [17] Y ボタンの状態(BOOL 値)
 * [18] －ボタンの状態(BOOL 値)
 * [19] Home ボタンの状態(BOOL 値)
 * [20] ＋ボタンの状態(BOOL 値)
 * [21] 十字キー・上ボタンの状態(BOOL 値)
 * [22] 十字キー・下ボタンの状態(BOOL 値)
 * [23] 十字キー・左ボタンの状態(BOOL 値)
 * [24] 十字キー・右ボタンの状態(BOOL 値)
 * [25] L ボタンの状態(BOOL 値)
 * [26] R ボタンの状態(BOOL 値)
 * [27] ZL ボタンの状態(BOOL 値)
 * [28] ZR ボタンの状態(BOOL 値)
 * [29] L ボタンのアナログ入力値
 * (最小値0，最大値100，PRO の場合は解放で0，押下で100)
 * [30] R ボタンのアナログ入力値
 * (最小値0，最大値100，PRO の場合は解放で0，押下で100)
 * [31] 左アナログスティックのX 方向入力値
 * (解放時0，最大まで左に傾けると-100，同右で100)
 * [32] 左アナログスティックのY 方向入力値
 * (解放時0，最大まで下に傾けると-100，同上で100)
 * [33] 右アナログスティックのX 方向入力値(-100～100)
 * (解放時0，最大まで左に傾けると-100，同右で100)
 * [34] 右アナログスティックのY 方向入力値(-100～100)
 * (解放時0，最大まで下に傾けると-100，同上で100)
 */

RTC::ReturnCode_t WiiClassicControllerTranslator::onExecute(RTC::UniqueId ec_id)
{
	BYTE data[22];
	double x, y, z, l_trigger, r_trigger, l_stick_x, l_stick_y, r_stick_x, r_stick_y;

	if(m_WiiDataIn.isNew())
	{
		m_WiiDataIn.read();
		for(int i= 0; i < 22; i++)
		{
			data[i] = m_WiiData.data[i];
		}

		wii.ParseButtons(data);
		wii.ParseAccel(data);
		wii.ParseAnalog(data);

		m_WiiConvert.data.length(35);
		m_WiiConvert.data[0] = wii.mWiiRemoteState.ButtonState.Up;
		m_WiiConvert.data[1] = wii.mWiiRemoteState.ButtonState.Right;
		m_WiiConvert.data[2] = wii.mWiiRemoteState.ButtonState.Down;
		m_WiiConvert.data[3] = wii.mWiiRemoteState.ButtonState.Left;
		m_WiiConvert.data[4] = wii.mWiiRemoteState.ButtonState.A;
		m_WiiConvert.data[5] = wii.mWiiRemoteState.ButtonState.B;
		m_WiiConvert.data[6] = wii.mWiiRemoteState.ButtonState.Plus;
		m_WiiConvert.data[7] = wii.mWiiRemoteState.ButtonState.Home;
		m_WiiConvert.data[8] = wii.mWiiRemoteState.ButtonState.Minus;
		m_WiiConvert.data[9] = wii.mWiiRemoteState.ButtonState.One;
		m_WiiConvert.data[10] = wii.mWiiRemoteState.ButtonState.Two;
		x = (double)(wii.mWiiRemoteState.AccelState.RawValues.X - wii.mWiiRemoteState.AccelCalibrationInfo.X0)*100
			/(double)(wii.mWiiRemoteState.AccelCalibrationInfo.XG - wii.mWiiRemoteState.AccelCalibrationInfo.X0);
		y = (double)(wii.mWiiRemoteState.AccelState.RawValues.Y - wii.mWiiRemoteState.AccelCalibrationInfo.Y0)*100
			/(double)(wii.mWiiRemoteState.AccelCalibrationInfo.YG - wii.mWiiRemoteState.AccelCalibrationInfo.Y0);
		z = (double)(wii.mWiiRemoteState.AccelState.RawValues.Z - wii.mWiiRemoteState.AccelCalibrationInfo.Z0)*100
			/(double)(wii.mWiiRemoteState.AccelCalibrationInfo.ZG - wii.mWiiRemoteState.AccelCalibrationInfo.Z0);

		m_WiiConvert.data[11] = (short)x;
		m_WiiConvert.data[12] = (short)y;
		m_WiiConvert.data[13] = (short)z;

		m_WiiConvert.data[14] = wii.mWiiClasicControllerState.ButtonState.A;
		m_WiiConvert.data[15] = wii.mWiiClasicControllerState.ButtonState.B;
		m_WiiConvert.data[16] = wii.mWiiClasicControllerState.ButtonState.X;
		m_WiiConvert.data[17] = wii.mWiiClasicControllerState.ButtonState.Y;
		m_WiiConvert.data[18] = wii.mWiiClasicControllerState.ButtonState.Minus;
		m_WiiConvert.data[19] = wii.mWiiClasicControllerState.ButtonState.Home;
		m_WiiConvert.data[20] = wii.mWiiClasicControllerState.ButtonState.Plus;
		m_WiiConvert.data[21] = wii.mWiiClasicControllerState.ButtonState.Up;
		m_WiiConvert.data[22] = wii.mWiiClasicControllerState.ButtonState.Down;
		m_WiiConvert.data[23] = wii.mWiiClasicControllerState.ButtonState.Left;
		m_WiiConvert.data[24] = wii.mWiiClasicControllerState.ButtonState.Right;
		m_WiiConvert.data[25] = wii.mWiiClasicControllerState.ButtonState.L;
		m_WiiConvert.data[26] = wii.mWiiClasicControllerState.ButtonState.R;
		m_WiiConvert.data[27] = wii.mWiiClasicControllerState.ButtonState.ZL;
		m_WiiConvert.data[28] = wii.mWiiClasicControllerState.ButtonState.ZR;

		l_trigger = (double)(wii.mWiiClasicControllerState.AnalogState.Left.Trigger - wii.mWiiClasicControllerState.CalibrationInfo.LeftTrigger.Min) * 100
			/ ((double)wii.mWiiClasicControllerState.CalibrationInfo.LeftTrigger.Max);
		r_trigger = (double)(wii.mWiiClasicControllerState.AnalogState.Right.Trigger - wii.mWiiClasicControllerState.CalibrationInfo.RightTrigger.Min) * 100
			/ ((double)wii.mWiiClasicControllerState.CalibrationInfo.RightTrigger.Max);
		l_stick_x = (double)(wii.mWiiClasicControllerState.AnalogState.Left.StickX - wii.mWiiClasicControllerState.CalibrationInfo.LeftStick.X0) * 100
			/ ((double)wii.mWiiClasicControllerState.CalibrationInfo.LeftStick.XMax / 2);
		l_stick_y = (double)(wii.mWiiClasicControllerState.AnalogState.Left.StickY - wii.mWiiClasicControllerState.CalibrationInfo.LeftStick.Y0) * 100
			/ ((double)wii.mWiiClasicControllerState.CalibrationInfo.LeftStick.YMax / 2);
		r_stick_x = (double)(wii.mWiiClasicControllerState.AnalogState.Right.StickX - wii.mWiiClasicControllerState.CalibrationInfo.RightStick.X0) * 100
			/ ((double)wii.mWiiClasicControllerState.CalibrationInfo.RightStick.XMax / 2);
		r_stick_y = (double)(wii.mWiiClasicControllerState.AnalogState.Right.StickY - wii.mWiiClasicControllerState.CalibrationInfo.RightStick.Y0) * 100
			/ ((double)wii.mWiiClasicControllerState.CalibrationInfo.RightStick.YMax / 2);

		m_WiiConvert.data[29] = (short)l_trigger;
		m_WiiConvert.data[30] = (short)r_trigger;
		m_WiiConvert.data[31] = (short)l_stick_x;
		m_WiiConvert.data[32] = (short)l_stick_y;
		m_WiiConvert.data[33] = (short)r_stick_x;
		m_WiiConvert.data[34] = (short)r_stick_y;


		if(m_debug)
		{
			system("cls");
			std::cout << "<<WiiRemoteState>>" << std::endl;
			std::cout << "↑:" << m_WiiConvert.data[0] << std::endl;
			std::cout << "→:" << m_WiiConvert.data[1] << std::endl;
			std::cout << "↓:" << m_WiiConvert.data[2] << std::endl;
			std::cout << "←:" << m_WiiConvert.data[3] << std::endl;
			std::cout << "Ａ:" << m_WiiConvert.data[4] << std::endl;
			std::cout << "Ｂ:" << m_WiiConvert.data[5] << std::endl;
			std::cout << "＋:" << m_WiiConvert.data[6] << std::endl;
			std::cout << "Home:" << m_WiiConvert.data[7] << std::endl;
			std::cout << "－:" << m_WiiConvert.data[8] << std::endl;
			std::cout << "１:" << m_WiiConvert.data[9] << std::endl;
			std::cout << "２:" << m_WiiConvert.data[10] << std::endl;
			std::cout << "X_Accel:" << m_WiiConvert.data[11] << std::endl;
			std::cout << "Y_Accel:" << m_WiiConvert.data[12] << std::endl;
			std::cout << "Z_Accel:" << m_WiiConvert.data[13] << std::endl;
			std::cout << std::endl;

			std::cout << "<<ClassicControllerState>>" << std::endl;
			std::cout << "↑:" << m_WiiConvert.data[21] << std::endl;
			std::cout << "→:" << m_WiiConvert.data[24] << std::endl;
			std::cout << "↓:" << m_WiiConvert.data[22] << std::endl;
			std::cout << "←:" << m_WiiConvert.data[23] << std::endl;
			std::cout << "Ａ:" << m_WiiConvert.data[14] << std::endl;
			std::cout << "Ｂ:" << m_WiiConvert.data[15] << std::endl;
			std::cout << "Ｘ:" << m_WiiConvert.data[16] << std::endl;
			std::cout << "Ｙ:" << m_WiiConvert.data[17] << std::endl;
			std::cout << "＋:" << m_WiiConvert.data[20] << std::endl;
			std::cout << "Home:" << m_WiiConvert.data[19] << std::endl;
			std::cout << "－:" << m_WiiConvert.data[18] << std::endl;
			std::cout << "Ｌ:" << m_WiiConvert.data[25] << std::endl;
			std::cout << "Ｒ:" << m_WiiConvert.data[26] << std::endl;
			std::cout << "ZL:" << m_WiiConvert.data[27] << std::endl;
			std::cout << "ZR:" << m_WiiConvert.data[28] << std::endl;
			std::cout << "L_Trigger:" << m_WiiConvert.data[29] << std::endl;
			std::cout << "R_Trigger:" << m_WiiConvert.data[30] << std::endl;
			std::cout << "L_Stick_X:" << m_WiiConvert.data[31] << std::endl;
			std::cout << "L_Stick_Y:" << m_WiiConvert.data[32] << std::endl;
			std::cout << "R_Stick_X:" << m_WiiConvert.data[33] << std::endl;
			std::cout << "R_Stick_Y:" << m_WiiConvert.data[34] << std::endl;
		}

		m_WiiConvertOut.write();
	}

	return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t WiiClassicControllerTranslator::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t WiiClassicControllerTranslator::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t WiiClassicControllerTranslator::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t WiiClassicControllerTranslator::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t WiiClassicControllerTranslator::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void WiiClassicControllerTranslatorInit(RTC::Manager* manager)
  {
    coil::Properties profile(wiiclassiccontrollertranslator_spec);
    manager->registerFactory(profile,
                             RTC::Create<WiiClassicControllerTranslator>,
                             RTC::Delete<WiiClassicControllerTranslator>);
  }
  
};


