// -*- C++ -*-
/*!
 * @file  WiiMotionPlusTranslator.cpp
 * @brief WiiMotionPlusTranslator
 * @date $Date$
 *
 * @author 〒135-8548
 * 東京都江東区豊洲3-7-5　芝浦工業大学　研究棟　11-Q-32
 * 水川(HRI)研究室
 * 代表者:水川　真   (Emai:mizukawa<at>sic.shibaura-it.ac.jp)
 * 作成者;田中　基雅(Email:m108081<at>shibaura-it.ac.jp)
 *          藤田　恒彦(Email:m109077<at>shibaura-it.ac.jp)
 *          鷹栖　尭大（Email:m109056<at>shibaura-it.ac.jp）
 *
 * WiiMotionPlusTranslatorについての著作権は，芝浦工業大学 水川研究室に帰属します
 * ．
 * 非営利目的利用は自由です．
 *
 * $Id$
 */

#include "WiiMotionPlusTranslator.h"

// Module specification
// <rtc-template block="module_spec">
static const char* wiimotionplustranslator_spec[] =
  {
    "implementation_id", "WiiMotionPlusTranslator",
    "type_name",         "WiiMotionPlusTranslator",
    "description",       "WiiMotionPlusTranslator",
    "version",           "1.0.0",
    "vendor",            "Shibaura Institute of Technology",
    "category",          "RemoteController",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.debug", "0",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
WiiMotionPlusTranslator::WiiMotionPlusTranslator(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_WiiDataIn("WiiData", m_WiiData),
    m_WiiConvertOut("WiiConvert", m_WiiConvert),
    m_WiiInfoPort("WiiInfo")

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
WiiMotionPlusTranslator::~WiiMotionPlusTranslator()
{
}


/*!
 * Configurationの初期化を行う
 */
RTC::ReturnCode_t WiiMotionPlusTranslator::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("WiiData", m_WiiDataIn);
  
  // Set OutPort buffer
  registerOutPort("WiiConvert", m_WiiConvertOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  m_WiiInfoPort.registerConsumer("WiiInfo", "WiiInfo", m_WiiInfo);
  
  // Set CORBA Service Ports
  registerPort(m_WiiInfoPort);
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("debug", m_debug, "0");
  
  // </rtc-template>
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t WiiMotionPlusTranslator::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t WiiMotionPlusTranslator::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t WiiMotionPlusTranslator::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*!
 * キャリブレーションデータを取得し，WiiRemoteAgentに送信する．
 */

RTC::ReturnCode_t WiiMotionPlusTranslator::onActivated(RTC::UniqueId ec_id)
{
	//キャリブレーションデータ格納用
	Calibration_var CalibList;
	//キャリブレーションデータの格納
	while(1){
		CalibList = m_WiiInfo->get_wii_mote_calib();
		if(CalibList[0] == 0x21){
			break;
		}
	}
	//Debug Write Line
	BYTE buff[22];
	for(int i = 0; i < 22 ; i++){
		buff[i] = CalibList[i];
	}

	m_MotionPlusConvert.SetWiiRemoteCalibration(buff);

	calibration_flag = false;
	ave_count = 0;

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t WiiMotionPlusTranslator::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*!
 * WiiRemoteAgentから生データが入力された時に，それを整形データに変換し，出力する
 * ．
 * 整形データはTimedShortSeq型で出力する．
 * データの内訳を以下に記す．
 * 尚，[0]から[13]までがWiiリモコン本体の情報，それ以降はWiiMotionPlusの情報となって
 * いる．
 * [0] 十字キー・上ボタンの状態(BOOL 値)
 * [1] 十字キー・右ボタンの状態(BOOL 値)
 * [2] 十字キー・下ボタンの状態(BOOL 値)
 * [3] 十字キー・左ボタンの状態(BOOL 値)
 * [4] A ボタンの状態(BOOL 値)
 * [5] B ボタンの状態(BOOL 値)
 * [6] ＋ボタンの状態(BOOL 値)
 * [7] Home ボタンの状態(BOOL 値)
 * [8] －ボタンの状態(BOOL 値)
 * [9] 1 ボタンの状態(BOOL 値)
 * [10] 2 ボタンの状態(BOOL 値)
 * [11] X軸方向の加速度
 * [12] Y軸方向の加速度
 * [13] Z軸方向の加速度
 * [14] MotionPlus のyaw の生データ
 * [15] MotionPlus のpitch の生データ
 * [16] MotionPlus のroll の生データ
 * [17] MotionPlus のyaw を速度に変換した値
 * [18] MotionPlus のpitch を速度に変換した値
 * [19] MotionPlus のroll を速度に変換した値
 */

RTC::ReturnCode_t WiiMotionPlusTranslator::onExecute(RTC::UniqueId ec_id)
{
	BYTE buff[255];

	if(m_WiiDataIn.isNew()){
		m_WiiDataIn.read();
		for(int i = 0; i < 22; i++){
			buff[i] = m_WiiData.data[i];
		}
		if(calibration_flag == true){
			m_MotionPlusConvert.ParseButtons(buff);
			m_MotionPlusConvert.ParseAccel(buff);
			m_MotionPlusConvert.ParseExtension(buff);

			//アウトポートへデータ格納
			m_WiiConvert.data.length(20);
			m_WiiConvert.data[0] = m_MotionPlusConvert.mWiiRemoteState.ButtonState.Up;
			m_WiiConvert.data[1] = m_MotionPlusConvert.mWiiRemoteState.ButtonState.Right;
			m_WiiConvert.data[2] = m_MotionPlusConvert.mWiiRemoteState.ButtonState.Down;
			m_WiiConvert.data[3] = m_MotionPlusConvert.mWiiRemoteState.ButtonState.Left;
			m_WiiConvert.data[4] = m_MotionPlusConvert.mWiiRemoteState.ButtonState.A;
			m_WiiConvert.data[5] = m_MotionPlusConvert.mWiiRemoteState.ButtonState.B;
			m_WiiConvert.data[6] = m_MotionPlusConvert.mWiiRemoteState.ButtonState.Plus;
			m_WiiConvert.data[7] = m_MotionPlusConvert.mWiiRemoteState.ButtonState.Home;
			m_WiiConvert.data[8] = m_MotionPlusConvert.mWiiRemoteState.ButtonState.Minus;
			m_WiiConvert.data[9] = m_MotionPlusConvert.mWiiRemoteState.ButtonState.One;
			m_WiiConvert.data[10] = m_MotionPlusConvert.mWiiRemoteState.ButtonState.Two;
			m_WiiConvert.data[11] = m_MotionPlusConvert.mWiiRemoteState.AccelState.Values.X;
			m_WiiConvert.data[12] = m_MotionPlusConvert.mWiiRemoteState.AccelState.Values.Y;
			m_WiiConvert.data[13] = m_MotionPlusConvert.mWiiRemoteState.AccelState.Values.Z;

			//raw data
			m_WiiConvert.data[14] = m_MotionPlusConvert.mMotionPlusState.gyro.yaw;
			m_WiiConvert.data[15] = m_MotionPlusConvert.mMotionPlusState.gyro.pitch;
			m_WiiConvert.data[16] = m_MotionPlusConvert.mMotionPlusState.gyro.roll;

			//deg data
			m_WiiConvert.data[17] = m_MotionPlusConvert.mMotionPlusState.gyro_speed.yaw_Speed;
			m_WiiConvert.data[18] = m_MotionPlusConvert.mMotionPlusState.gyro_speed.pitch_Speed;
			m_WiiConvert.data[19] = m_MotionPlusConvert.mMotionPlusState.gyro_speed.roll_Speed;

			m_WiiConvertOut.write();

			if(m_debug)
			{
				system("cls");
				std::cout << "<<WiiRemoteState>>" << std::endl;
				std::cout << "↑:" << m_WiiConvert.data[0] << std::endl;
				std::cout << "→:" << m_WiiConvert.data[1] << std::endl;
				std::cout << "↓:" << m_WiiConvert.data[2] << std::endl;
				std::cout << "←:" << m_WiiConvert.data[3] << std::endl;
				std::cout << "Ａ:" << m_WiiConvert.data[4] << std::endl;
				std::cout << "Ｂ:" << m_WiiConvert.data[5] << std::endl;
				std::cout << "＋:" << m_WiiConvert.data[6] << std::endl;
				std::cout << "Home:" << m_WiiConvert.data[7] << std::endl;
				std::cout << "－:" << m_WiiConvert.data[8] << std::endl;
				std::cout << "１:" << m_WiiConvert.data[9] << std::endl;
				std::cout << "２:" << m_WiiConvert.data[10] << std::endl;
				std::cout << "X_Accel:" << m_WiiConvert.data[11] << std::endl;
				std::cout << "Y_Accel:" << m_WiiConvert.data[12] << std::endl;
				std::cout << "Z_Accel:" << m_WiiConvert.data[13] << std::endl;
				std::cout << std::endl;

				std::cout << "<<Motion Plus Raw Data>>" << std::endl;
				std::cout << "yaw_ :" << m_WiiConvert.data[14] << std::endl;
				std::cout << "pitch:" << m_WiiConvert.data[15] << std::endl;
				std::cout << "roll :" << m_WiiConvert.data[16] << std::endl;
				std::cout << std::endl;

				std::cout << "<<Motion Plus Speed Data>>" << std::endl;
				std::cout << "yaw  :" << m_WiiConvert.data[17] << std::endl;
				std::cout << "pitch:" << m_WiiConvert.data[18] << std::endl;
				std::cout << "roll :" << m_WiiConvert.data[19] << std::endl;

			}
		}else{
			//モーションプラスのキャリブレーションが解析途中のため平均を利用して設定
			//平均にする回数の設定(コンフィギュレーションに設定できたらする)
			if( ave_count == 10){
				calibration_flag = true;
				std::cout << "Calibration End" << std::endl;
				std::cout << "yaw0  : " << m_MotionPlusConvert.mMotionPlusState.AveCalib.yaw0 << std::endl;
				std::cout << "pitch0: " << m_MotionPlusConvert.mMotionPlusState.AveCalib.pitch0 << std::endl;
				std::cout << "roll0 : " << m_MotionPlusConvert.mMotionPlusState.AveCalib.roll0 << std::endl;

			}else{

				m_MotionPlusConvert.ParseMotionPlusCalib(buff,10);
				ave_count++;
			}
		}
	}
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t WiiMotionPlusTranslator::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t WiiMotionPlusTranslator::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t WiiMotionPlusTranslator::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t WiiMotionPlusTranslator::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t WiiMotionPlusTranslator::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void WiiMotionPlusTranslatorInit(RTC::Manager* manager)
  {
    coil::Properties profile(wiimotionplustranslator_spec);
    manager->registerFactory(profile,
                             RTC::Create<WiiMotionPlusTranslator>,
                             RTC::Delete<WiiMotionPlusTranslator>);
  }
  
};


