// -*- C++ -*-
/*!
 * @file  WiiNunchakuTranslator.h
 * @brief WiiNunchakuTranslator
 * @date  $Date$
 *
 * @author 〒135-8548
 * 東京都江東区豊洲3-7-5　芝浦工業大学　研究棟　11-Q-32
 * 水川(HRI)研究室
 * 代表者:水川　真   (Emai:mizukawa<at>sic.shibaura-it.ac.jp)
 * 作成者;田中　基雅(Email:m108081<at>shibaura-it.ac.jp)
 *          藤田　恒彦(Email:m109077<at>shibaura-it.ac.jp)
 *          鷹栖　尭大（Email:m109056<at>shibaura-it.ac.jp）
 *
 * WiiNunchakuTranslatorについての著作権は，芝浦工業大学 水川研究室に帰属します．
 * 非営利目的利用は自由です．
 *
 * $Id$
 */

#ifndef WIINUNCHAKUTRANSLATOR_H
#define WIINUNCHAKUTRANSLATOR_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>

// Service implementation headers
// <rtc-template block="service_impl_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">
#include "WiiInfoStub.h"
#include "WiiNunchakuConvert.h"

// </rtc-template>

using namespace RTC;

/*!
 * @class WiiNunchakuTranslator
 * @brief WiiNunchakuTranslator
 *
 */
class WiiNunchakuTranslator
  : public RTC::DataFlowComponentBase
{
 public:
  /*!
   * @brief constructor
   * @param manager Maneger Object
   */
  WiiNunchakuTranslator(RTC::Manager* manager);

  /*!
   * @brief destructor
   */
  ~WiiNunchakuTranslator();

  // <rtc-template block="public_attribute">
  
  // </rtc-template>

  // <rtc-template block="public_operation">
  
  // </rtc-template>

  /***
   * Configurationの初期化を行う
   *
   * The initialize action (on CREATED->ALIVE transition)
   * formaer rtc_init_entry() 
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onInitialize();

  /***
   * 
   *
   * The finalize action (on ALIVE->END transition)
   * formaer rtc_exiting_entry()
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onFinalize();

  /***
   *
   * The startup action when ExecutionContext startup
   * former rtc_starting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  /***
   *
   * The shutdown action when ExecutionContext stop
   * former rtc_stopping_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  /***
   * キャリブレーションデータを取得し，WiiRemoteAgentに送信する．
   *
   * The activated action (Active state entry action)
   * former rtc_active_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre WiiリモコンにNunchakuを接続している事
   * 
   */
   virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  /***
   *
   * The deactivated action (Active state exit action)
   * former rtc_active_exit()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  /***
   * WiiRemoteAgentから生データが入力された時に，それを整形データに変換し，出力す
   * る．
   * 整形データはTimedShortSeq型で出力する．
   * データの内訳を以下に記す．
   * 尚，[0]から[13]までがWiiリモコン本体の情報，それ以降はNunchakuの情報となっている
   * ．
   * [0] 十字キー・上ボタンの状態(BOOL 値)
   * [1] 十字キー・右ボタンの状態(BOOL 値)
   * [2] 十字キー・下ボタンの状態(BOOL 値)
   * [3] 十字キー・左ボタンの状態(BOOL 値)
   * [4] A ボタンの状態(BOOL 値)
   * [5] B ボタンの状態(BOOL 値)
   * [6] ＋ボタンの状態(BOOL 値)
   * [7] Home ボタンの状態(BOOL 値)
   * [8] －ボタンの状態(BOOL 値)
   * [9] 1 ボタンの状態(BOOL 値)
   * [10] 2 ボタンの状態(BOOL 値)
   * [11] X軸方向の加速度
   * [12] Y軸方向の加速度
   * [13] Z軸方向の加速度
   * [14] C ボタンの状態(BOOL 値)
   * [15] Z ボタンの状態(BOOL 値)
   * [16] アナログスティックのX 方向入力値
   * (解放時0，最大まで左に傾けると-100，同右で100)
   * [17] アナログスティックのY 方向入力値
   * (解放時0，最大まで左に傾けると-100，同右で100)
   * [18] X軸方向の加速度
   * [19] Y軸方向の加速度
   * [20] Z軸方向の加速度
   *
   * The execution action that is invoked periodically
   * former rtc_active_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  /***
   *
   * The aborting action when main logic error occurred.
   * former rtc_aborting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  /***
   *
   * The error action in ERROR state
   * former rtc_error_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  /***
   *
   * The reset action that is invoked resetting
   * This is same but different the former rtc_init_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  /***
   *
   * The state update action that is invoked after onExecute() action
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  /***
   *
   * The action that is invoked when execution context's rate is changed
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // <rtc-template block="protected_attribute">
  
  // </rtc-template>

  // <rtc-template block="protected_operation">
  
  // </rtc-template>

  // Configuration variable declaration
  // <rtc-template block="config_declare">
  /*!
   * コンソール画面に整形データを表示するかどうかを選択するためのConfiguration
   * - Name:  debug
   * - DefaultValue: 0
   */
  bool m_debug;
  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  TimedOctetSeq m_WiiData;
  /*!
   * WiiRemoteAgentから，WiiリモコンとNunchakuの情報を受け取る．
   * - Type: TimedOctetSeq
   * - Number: 22Byte
   */
  InPort<TimedOctetSeq> m_WiiDataIn;
  
  // </rtc-template>


  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  TimedShortSeq m_WiiConvert;
  /*!
   * Wiiリモコン本体，Nunchakuの情報を整形したデータを出力する．
   * - Type: TimedShortSeq
   * - Number: 21
   */
  OutPort<TimedShortSeq> m_WiiConvertOut;
  
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  /*!
   * WiiRemoteAgentにWiiリモコン本体及びNunchakuのキャリブレーションデータを要求
   * する．
   */
  RTC::CorbaPort m_WiiInfoPort;
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  /*!
   */
  RTC::CorbaConsumer<WiiInfo> m_WiiInfo;
  
  // </rtc-template>

 private:
  WiiNunchakuConvert wii;
  // <rtc-template block="private_attribute">
  
  // </rtc-template>

  // <rtc-template block="private_operation">
  
  // </rtc-template>

};


extern "C"
{
  DLL_EXPORT void WiiNunchakuTranslatorInit(RTC::Manager* manager);
};

#endif // WIINUNCHAKUTRANSLATOR_H
