// -*-C++-*-
/*!
 * @file  WiiInfoSVC_impl.h
 * @brief Service implementation header of WiiInfo.idl
 *
 * @author 〒135-8548
 * 東京都江東区豊洲3-7-5　芝浦工業大学　研究棟　11-Q-32
 * 水川(HRI)研究室
 * 代表者:水川　真   (Emai:mizukawa<at>sic.shibaura-it.ac.jp)
 * 作成者;田中　基雅(Email:m108081<at>shibaura-it.ac.jp)
 *           藤田　恒彦(Email:m109077<at>shibaura-it.ac.jp)
 *           鷹栖　尭大（Email:m109056<at>shibaura-it.ac.jp）
 *
 * Microsoftより提供されているWindows Driver Kitのライブラリを除き，芝浦工業大学
 * 水川研究室に帰属します．
 * 非営利目的利用は自由です．
 *
 */

#include "WiiInfoSkel.h"

#ifndef WIIINFOSVC_IMPL_H
#define WIIINFOSVC_IMPL_H
 
/*!
 * @class WiiInfoSVC_impl
 * Example class implementing IDL interface WiiInfo
 */
class WiiInfoSVC_impl
 : public virtual POA_WiiInfo,
   public virtual PortableServer::RefCountServantBase
{
 private:
   // Make sure all instances are built on the heap by making the
   // destructor non-public
   //virtual ~WiiInfoSVC_impl();

 public:
  /*!
   * @brief standard constructor
   */
   WiiInfoSVC_impl();
  /*!
   * @brief destructor
   */
   virtual ~WiiInfoSVC_impl();

   //拡張コントローラのキャリブレーション
   Calibration* get_wii_ext_calib();
   //バランスボードのキャリブレーション
   Calibration* get_balance_calib();
   //Wiiリモコンのキャリブレーション
   Calibration* get_wii_mote_calib();
   //バッテリ情報
   CORBA::Octet get_battery_value();
   //拡張コントローラの情報
   Extension* get_wii_ext_state();

   //Wiiリモコンのキャリブレーションデータの格納用
   Calibration m_valueWiiRemoteCalibration;
   //拡張コントローラのキャリブレーション格納用
   Calibration m_valueWiiExtensionCalibration;
   //バランスボードのキャリブレーションデータ格納用
   Calibration m_valueWiiBalanceCalibration;
   //バッテリデータの格納用
   CORBA::Octet m_valueBattery;
   //拡張コントローラの情報格納用
   Extension m_valueWiiState;

};



#endif // WIIINFOSVC_IMPL_H


